
Game = Core.class(Sprite)

function Game:init()
  GAME = self -- make this scene a global variable
  
  self:startUp()
  
  self:addEventListener("enterBegin", self.onTransitionInBegin, self)
  self:addEventListener("enterEnd", self.onTransitionInEnd, self)
  self:addEventListener("exitBegin", self.onTransitionOutBegin, self)
  self:addEventListener("exitEnd", self.onTransitionOutEnd, self)
end

function Game:onTransitionInBegin() -- INITIALIZE VARIABLES HERE
end

function Game:onTransitionInEnd() -- ADD LISTENERS, START LOGIC
  self:addEventListener(Event.ENTER_FRAME, self.update, self)
  self:addEventListener(Event.TOUCHES_BEGIN, self.onTouchesBegin, self)
--  self:addEventListener(Event.TOUCHES_MOVE, self.onTouchesMove, self)
--  self:addEventListener(Event.TOUCHES_END, self.onTouchesEnd, self)
--  self:addEventListener(Event.TOUCHES_CANCEL, self.onTouchesCancel, self)

  application:setBackgroundColor(0xddf8ff)
end

function Game:onTransitionOutBegin() -- STOP TIMERS, REMOVE LISTENERS
  self:removeEventListener(Event.ENTER_FRAME, self.update, self)
  self:removeEventListener(Event.TOUCHES_BEGIN, self.onTouchesBegin, self)
--  self:removeEventListener(Event.TOUCHES_MOVE, self.onTouchesMove, self)
--  self:removeEventListener(Event.TOUCHES_END, self.onTouchesEnd, self)
--  self:removeEventListener(Event.TOUCHES_CANCEL, self.onTouchesCancel, self)
end

function Game:onTransitionOutEnd() -- GONE FROM STAGE, CLEANUP
  self:removeEventListener("enterBegin", self.onTransitionInBegin, self)
  self:removeEventListener("enterEnd", self.onTransitionInEnd, self)
  self:removeEventListener("exitBegin", self.onTransitionOutBegin, self)
  self:removeEventListener("exitEnd", self.onTransitionOutEnd, self)
end

-- spawn a new gear at a random position
function Game:newGear()
  self.gears:addChild(Gear.new(self.gearTex, 0, 1 + math.random(), 50 + (math.random() * 270), -50))
end

function Game:startUp()
  local gears = Sprite.new() -- our group of gears
  self:addChild(gears)
  self.gears = gears
  
  local playerProjectiles = Sprite.new() -- our group of player projectiles
  self:addChild(playerProjectiles)
  self.playerProjectiles = playerProjectiles
  
  local gearText = TextField.new(nil, '') -- debug text for gears
  gearText:setTextColor(0x000000)
  gearText:setScale(2)
  gearText:setPosition(4, 48)
  self.gearText = gearText
  self:addChild(gearText)  
  
  -- some debug text to show how many projectiles are on screen
  local projectileText = TextField.new(nil, '')
  projectileText:setTextColor(0x000000)
  projectileText:setScale(2)
  projectileText:setPosition(4, 24)
  self.projectileText = projectileText
  self:addChild(projectileText)
  
  -- only need to get these only once
  self.fireTex = Texture.new('fire_1.png') -- texture to use for player projectiles
  self.gearTex = Texture.new('gear_1.png') -- texture to use for gear
  
  self:reset()

  -- spawn 5 gears to start with
  for i = 1, 5 do
    self:newGear()
  end
end

-- reset all game stuff before starting a new game
function Game:reset()
  self.paused = true
  self.counter = 0 -- gear spawn timer
  
  -- dispose of all gears
  local objects = self.gears
  for i = objects:getNumChildren(), 1, -1 do
    local object = objects(i)
    object:removeFromParent()
  end

  -- dispose of all player projectiles
  objects = self.playerProjectiles
  for i = objects:getNumChildren(), 1, -1 do
    local object = objects(i)
    object:removeFromParent()
  end
  
  self.paused = false
end

-- update every frame
function Game:update(e)
  if self.paused then return end -- don't process anything if the game is paused
  
  local dt = e.deltaTime
  
  -- spawn a new gear every 2 seconds
  local counter = self.counter + dt
  if counter >= 1 then
    counter -= 1
    self:newGear()
  end
  self.counter = counter
  
  -- process all player projectiles
  local objects = self.playerProjectiles
  for i = objects:getNumChildren(), 1, -1 do
    local object = objects:getChildAt(i)
    object:update(dt)
  end
  self.projectileText:setText('projectiles: ' .. objects:getNumChildren())

  -- process all gears
  objects = self.gears
  for i = objects:getNumChildren(), 1, -1 do
    local object = objects:getChildAt(i)
    object:update(dt)
  end
  self.gearText:setText('gears: ' .. objects:getNumChildren())
end

-- when the screen is touched spawn a new player projectile
function Game:onTouchesBegin(e)
  local tx, ty = e.touch.x, e.touch.y
  
  if ty >= 200 then
    self.playerProjectiles:addChild(Fire.new(self.fireTex, 0, -3, tx, ty))
  end
end

--function Game:onTouchesMove(e)
--end

--function Game:onTouchesEnd(e)
--end

--function Game:onTouchesCancel(e)
--end
